/* This code is distributed under the terms of Gnu Publice License */
/* The author is Przemek Borys <pborys@dione.ids.pl> */
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "utils.h"
#include "variables.h"
#include "dim.h"

#define USE_SPACING

struct Tarray
  {
    struct Tgraph **down;	/* the first two fields are identical */
    Tdim dim;			/* as in Tgraph--important for deallocation */
    int children;		/* the number of cells */
    int rows;
    int cols;
    int *rowy;
    int *colx;
  };

char *
findArrayDelimiter (char *txt)
{
  int len = strlen (txt);
  int i;
  for (i = 0; i < len; i++)
    {
      if (txt[i] == '\\')
	{
	  if (strncmp (txt + i, "\\begin", 6) == 0)	/* skip nested parts */
	    i += 6 + getbegin_endEnd (txt + i + 1) - (txt + i);
	}
      if ((txt[i] == '&') || ((txt[i] == '\\') && (txt[i + 1] == '\\')))
	return txt + i;
    }
  return txt + i;		/* no delimiter has been found */
}

int
dimArray (char *found, char **Gpos, Tdim * Our, struct Tgraph *graph)
{
#define gpos (*Gpos)
#define our (*Our)
  char *start = found + 6 + 7, *end, *tmp = getbegin_endEnd (found + 1);
  Tdim out;

  char **cells = (char **) malloc (sizeof (char *));
  int ncells = 0;
  int rows = 0, cols = 0;
  int curcols = 0;
  int i, j;

  if (tmp)
    *tmp = 0;
  else
    {
      fprintf (stderr, "Could not find matching \\end in array %s\n", found);
      exit (1);
    }

  *gpos = 1;			/* mark the array as #1#10 */
  gpos++;
  *gpos = 10;
  gpos++;
  *gpos = 0;

  while (1)
    {
      end = findArrayDelimiter (start);
      cells = (char **) realloc (cells, (ncells + 1) * (sizeof (char *)));
      cells[ncells] = (char *) malloc (end - start + 1);
      strncpy (cells[ncells], start, end - start);
      cells[ncells][end - start] = 0;	/* terminate the string */
      ncells++;
      if (*end == '&')
	{
	  start = end + 1;
	  curcols++;
	}
      else if (*end == '\\')
	{
	  curcols++;
	  start = end + 2;
	  if ((cols != 0) && (curcols != cols))
	    {
	      fprintf (stderr, "Bad number of collumns in array %s\n", found);
	      exit (1);
	    }
	  cols = curcols;
	  curcols = 0;
	  rows++;
	}
      else if (*end == 0)
	break;
    }
  rows++;
  /* now we have cells separated, and we know the number of cols/rows */
  

  if (graph->children == 0)
    graph->down = (struct Tgraph **) malloc (sizeof (struct Tgraph *));
  else
    graph->down = (struct Tgraph **) realloc (graph->down, sizeof (struct Tgraph **) * (graph->children + 1));

  graph->down[graph->children] = (struct Tgraph *) malloc (sizeof (struct Tarray));
  graph->children++;
#define Array ((struct Tarray*)graph->down[graph->children-1])
  Array->children=ncells;
  Array->rows = rows;
  Array->cols = cols;
  Array->rowy = (int *) calloc (rows, sizeof (int));
  Array->colx = (int *) calloc (cols, sizeof (int));
  Array->down = (struct Tgraph **) malloc (sizeof (struct Tgraph *) * ncells);
  for (i = 0; i < ncells; i++)
    {
      int whichrow = i / cols;
      int whichcol = i - whichrow * cols;
      Array->down[i] = (struct Tgraph *) malloc (sizeof (struct Tgraph));
      Array->down[i]->children = 0;
      Array->down[i]->up = graph;
      out = dim (cells[i], Array->down[i]);
      if (out.x > Array->colx[whichcol])
	Array->colx[whichcol] = out.x;
      if (out.y > Array->rowy[whichrow])
	Array->rowy[whichrow] = out.y;
    }
  Array->dim.x = 0;
  for (i = 0; i < cols; i++)
    Array->dim.x += Array->colx[i];
  Array->dim.y = 0;
  for (i = 0; i < rows; i++)
    Array->dim.y += Array->rowy[i];

#ifdef USE_SPACING
  Array->dim.y += Array->rows - 1;
  Array->dim.x += Array->cols - 1;
#endif /* USE_SPACING */

  Array->dim.baseline = Array->dim.y / 2;

  our.x += Array->dim.x;
  if (our.baseline < Array->dim.y / 2)
    {
      our.y += Array->dim.y / 2 - our.baseline;
      our.baseline = Array->dim.y / 2;
    }
  if (our.y < Array->dim.y)
    our.y = Array->dim.y;
#undef Array
  *tmp = '\\';			/* restore original value */
  return (tmp + 3 + 7) - found;
#undef gpos
#undef our
}

void
drawArray (int *Kid, int *Curx, int *Cury, char ***screen, struct Tgraph *graph)
{
#define kid (*Kid)
#define curx (*Curx)
#define Array ((struct Tarray*)graph->down[kid])
  int width = Array->dim.x;
  int cury = (*Cury) - ((Array->dim.y - 1) - Array->dim.baseline);
  int x = 0, y = 0, curitem = 0;
  int i, j;
  for (i = 0; i < Array->rows; i++)
    {
      for (j = 0; j < Array->cols; j++)
	{
	  drawInternal (screen, Array->down[curitem], curx + x +
			(Array->colx[j] - Array->down[curitem]->dim.x) / 2,	/* center in the cell */
			cury + y +
			(Array->rowy[i] - Array->down[curitem]->dim.y) / 2);	/* center in the cell */
	  curitem++;
	  x += Array->colx[j];
#ifdef USE_SPACING
	  x++;
#endif
	}
      y += Array->rowy[i];
#ifdef USE_SPACING
      y++;
#endif
      x = 0;
    }

  curx += Array->dim.x;
  kid++;
#undef Array
}
